 MODULE vdif_cd_mod
 
!======================================================================================================================!
! Subject:
!---------
!   Module used to compute the exchange coefficient in the surface layers Cd; Ch
!----------------------------------------------------------------------------------------------------------------------!
! Reference:
!-----------
!  Colaïtis, A., Spiga, A., Hourdin, F., Rio, C., Forget, F., and Millour, E. (2013), A thermal plume model for the Martian convective boundary layer, J. Geophys. Res. Planets, 118, 1468–1487, doi:10.1002/jgre.20104.
!
!======================================================================================================================!

IMPLICIT NONE

CONTAINS

   SUBROUTINE vdif_cd(ngrid,nlay,pz0,pg,pz,pu,pv,wstar,pts,ph,pcdv,pcdh)
   
   use comcstfi_h
   use turb_mod, only: turb_resolved


   IMPLICIT NONE
      
!=======================================================================
!
!   Subject: computation of the surface drag coefficient using the
!   -------  approch developed by Loui for ECMWF.
!
!   Author: Frederic Hourdin  15 /10 /93
!   Modified by : Arnaud Colaitis 03/08/11; rewritten by LL to switch to F90
!   -------
!
!   Arguments:
!   ----------
!
!   inputs:
!   ------
!     ngrid            size of the horizontal grid
!     pg               gravity (m s -2)
!     pz(ngrid,nlay)   height of layers
!     pu(ngrid,nlay)   u component of the wind
!     pv(ngrid,nlay)   v component of the wind
!     pts(ngrid)       surface temperature
!     ph(ngrid)        potential temperature T*(p/ps)^kappa
!
!   outputs:
!   --------
!     pcdv(ngrid)      Cd for the wind
!     pcdh(ngrid)      Cd for potential temperature
!
!=======================================================================


!-----------------------------------------------------------------------
!   Declarations:
!   -------------

#include "callkeys.h"


!   Arguments:

!   Inputs:
!   ----------
      INTEGER, INTENT(IN) :: ngrid,nlay                     ! Number of points in the physical grid and atmospheric grid
      REAL, INTENT(IN) :: pz0(ngrid)                        ! Surface Roughness (m)
      REAL, INTENT(IN) :: pg,pz(ngrid,nlay)                 ! Mars gravity (m/s^2)
      REAL, INTENT(IN) :: pu(ngrid,nlay),pv(ngrid,nlay)     ! Zonal and Meriditionnal  winds (m/s)
      REAL, INTENT(IN) :: pts(ngrid),ph(ngrid,nlay)         ! Surface Temperature and atmospheric temperature (K)
      REAL, INTENT(IN) :: wstar(ngrid)                      ! Vertical velocity due to turbulence (m/s)
      
!   Outputs:
!   ----------
      REAL, INTENT(OUT) :: pcdv(ngrid),pcdh(ngrid)          ! momentum and heat drag coefficient (1)

!   Local:
!   ------

      INTEGER ig        ! Loop variable

      REAL karman,nu    ! Von Karman constant and fluid kinematic viscosity
      
      LOGICAL firstcal
      DATA karman,nu/.41,0.001/
      DATA firstcal/.true./
      SAVE karman,nu

!$OMP THREADPRIVATE(karman,nu)

      REAL rib(ngrid)        ! Bulk Richardson number (1)
      REAL fm(ngrid)         ! stability function for momentum (1)
      REAL fh(ngrid)         ! stability function for heat (1)

! Formalism for stability functions  fm= 1/phim^2; fh = 1/(phim*phih) where
! phim = 1+betam*zeta   or   (1-bm*zeta)**am
! phih = alphah + betah*zeta    or   alphah(1.-bh*zeta)**ah
! ah and am are assumed to be -0.25 and -0.5 respectively
! lambda is an intermediate variable to simplify the expression
      REAL betam, betah, alphah, bm, bh, lambda

      REAL pz0t              ! initial thermal roughness length z0t for the iterative scheme (m)
      REAL ric               ! critical richardson number (1)
      REAL reynolds(ngrid)   ! Reynolds number (1)
      REAL prandtl(ngrid)    ! Prandtl number  (1)
      REAL pz0tcomp(ngrid)   ! computed z0t during the iterative scheme (m)
      REAL ite               ! Number of iteration (1)
      REAL itemax            ! Maximum number of iteration (1)
      REAL residual          ! Residual between pz0t and pz0tcomp (m)
      REAL tol_iter          ! Tolerance for the residual to ensure convergence (1=

      REAL zu2(ngrid)        ! Near surface winds (m/s)
      
      REAL cdn(ngrid)        ! neutral momentum  drag coefficient (1)
      REAL chn(ngrid)        ! neutral heat  drag coefficient (1)
      REAL z1z0,z1z0t        ! ratios z1/z0 and z1/z0T
      REAL z1,zcd0           ! Neutral roughness (m) and Cd/Ch coefficient when call richls is not called 
      
!    Code:
!    --------

! Initialisation
      itemax= 10 
      tol_iter =  0.01
      reynolds(:) = 0.
      pz0t = 0.
      pz0tcomp(:) = 0.1*pz0(:)
      rib(:) = 0.
      pcdv(:) = 0.
      pcdh(:) = 0.
      
! this formulation assumes alphah=1., implying betah=betam
! We use Dyer et al. parameters, as they cover a broad range of Richardson numbers :
      bm = 16.           
      bh = 16.            
      alphah = 1.
      betam = 5.         
      betah = 5.         
      lambda = (sqrt(bh/bm))/alphah
      
      ric = betah/(betam**2)

      IF(.not.callrichsl) then
! Original formulation as in LMDZ Earth:  Cd = Ch = (kappa/(ln(1+z1/z0)))^2
! NB: In forget et al., 1999, it's Cd = Ch = (kappa/(ln(z1/z0)))^2
         DO ig=1,ngrid
            z1=1.E+0 + pz(ig,1)/pz0(ig)
            zcd0=karman/log(z1)
            zcd0=zcd0*zcd0
            pcdv(ig)=zcd0
            pcdh(ig)=zcd0
         ENDDO
      ELSE  
! We follow the parameterization from Colaitis et al., 2013 (supplementary material)
         DO ig=1,ngrid
            ite = 0.
            residual = abs(pz0tcomp(ig)-pz0t) 
            z1z0=pz(ig,1)/pz0(ig)
            cdn(ig)=karman/log(z1z0)
            cdn(ig)=cdn(ig)*cdn(ig)
            
            DO WHILE((residual .gt. tol_iter*pz0(ig)) .and.  (ite .lt. itemax)) 
! Computations of z0T; iterated until z0T converges  
               pz0t = pz0tcomp(ig)
               z1z0t=pz(ig,1)/pz0t
               chn(ig)=cdn(ig)*log(z1z0)/log(z1z0t) 
               IF ((pu(ig,1) .ne. 0.) .or. (pv(ig,1) .ne. 0.)) then                              
                  zu2(ig) = pu(ig,1)*pu(ig,1)+pv(ig,1)*pv(ig,1) + (log(1.+0.7*wstar(ig) + 2.3*wstar(ig)**2))**2 ! Near surface winds account for buoyancy 
                  IF(turb_resolved) zu2(ig)=MAX(zu2(ig),1.) 
! Richardson number formulation proposed by D.E. England et al. (1995)
                  rib(ig) = (pg/pts(ig))*sqrt(pz(ig,1)*pz0(ig))*(((log(pz(ig,1)/pz0(ig)))**2)/(log(pz(ig,1)/pz0t)))*(ph(ig,1)-pts(ig))/zu2(ig)
                ELSE
                   print*,'warning, infinite Richardson at surface'
                   print*,pu(ig,1),pv(ig,1)
                   rib(ig) = ric         
                ENDIF
        
! Compute the stability functions fm; fh depending on the stability of the surface layer, from D.E. England et al. (95)

               ! STABLE BOUNDARY LAYER :
               IF (rib(ig) .gt. 0.) THEN
                  prandtl(ig) = 1.
                  IF(rib(ig) .lt. ric) THEN
               ! Assuming alphah=1. and bh=bm for stable conditions :
                     fm(ig) = ((ric-rib(ig))/ric)**2
                     fh(ig) = ((ric-rib(ig))/ric)**2
                   ELSE
               ! For Ri>Ric, we consider Ri->Infinity => no turbulent mixing at surface
                     fm(ig) = 1.
                     fh(ig) = 1.
                   ENDIF
               ! UNSTABLE BOUNDARY LAYER :
               ELSE
                  fm(ig) = sqrt(1.-lambda*bm*rib(ig))
                  fh(ig) = (1./alphah)*((1.-lambda*bh*rib(ig))**0.5)*(1.-lambda*bm*rib(ig))**0.25
                  prandtl(ig) = alphah*((1.-lambda*bm*rib(ig))**0.25)/((1.-lambda*bh*rib(ig))**0.5)
               ENDIF
              ! Recompute the reynolds and z0t 
               reynolds(ig) = karman*sqrt(fm(ig))*sqrt(zu2(ig))*pz0(ig)/(log(z1z0)*nu)
               pz0tcomp(ig) = pz0(ig)*exp(-karman*7.3*(reynolds(ig)**0.25)*(prandtl(ig)**0.5)+5*karman)
               residual = abs(pz0t-pz0tcomp(ig))
               ite = ite+1
            ENDDO  ! of while
            
! Compute the coefficients Cdv; Cdh : neutral coefficient x stability functions       
         pcdv(ig)=cdn(ig)*fm(ig)
         pcdh(ig)=chn(ig)*fh(ig)         
         pz0t = 0. ! for next grid point
         ENDDO ! of ngrid

      ENDIF !of if call richardson surface layer

      RETURN
      
 END SUBROUTINE vdif_cd

END MODULE vdif_cd_mod
      
      
      
      
      
      
      
      
       

